from setuphelpers import *

"""
Installation procedures:
https://mozilla.github.io/policy-templates/
https://mozilla.github.io/policy-templates/#extensionsettings

{
    "policies": {
        "ExtensionSettings": {
            "uBlock0@raymondhill.net": {
                "installation_mode": "force_installed",
                "install_url": "file://C:\\Program Files\\Mozilla Firefox\\distribution\\ublock_origin-1.57.2.xpi"
            }
        }
    }
}

"""
policies_file = "policies.json"
addon_id = "uBlock0@raymondhill.net"
extension_file_contains = "ublock_origin-"
extension_dict = {"policies": {"ExtensionSettings": {addon_id: {}}}}


def get_firefox_policies_dirs():
    detected_apps = installed_softwares("Firefox")
    firefox_policies_dirs = []
    if get_os_name() == "Linux":
        firefox_policies_dirs.append("/etc/firefox/policies")
    else:
        if not detected_apps:
            error("Firefox is not installed on this machine.")
        for ffox in detected_apps:
            if get_os_name() == "Windows":
                firefox_policies_dirs.append(makepath(ffox["install_location"], "distribution"))
            if get_os_name() == "Darwin":
                firefox_policies_dirs.append(makepath(ffox["install_location"], "Contents", "Resources", "distribution"))

    return firefox_policies_dirs


def install():
    bin_name = glob.glob("*.xpi")[0]

    # Interact differently with Luti
    if params.get("running_as_luti", False):
        WAPT.install("tis-firefox")

    for firefox_policies_dir in get_firefox_policies_dirs():
        policies_path = makepath(firefox_policies_dir, policies_file)
        killalltasks(ensure_list(control.impacted_process))
        if get_os_name() == "Darwin":
            run("sudo defaults write /Library/Preferences/org.mozilla.firefox EnterprisePoliciesEnabled -bool TRUE")
        mkdirs(firefox_policies_dir)

        # Copying extension_file
        extension_path = makepath(firefox_policies_dir, bin_name)
        print(f"Copying: {extension_path}")
        filecopyto(bin_name, firefox_policies_dir)

        # Prepare policies.json
        extension_dict["policies"]["ExtensionSettings"][addon_id]["install_url"] = "file://" + extension_path
        extension_dict["policies"]["ExtensionSettings"][addon_id]["installation_mode"] = "force_installed"

        # Write policies.json
        if isfile(policies_path):
            app_policies_dict = json_load_file(policies_path)
        else:
            app_policies_dict = {}
        merged_app_policies_dict = merge_dicts(app_policies_dict, extension_dict)
        print(f"Updating Firefox configuration to: {policies_path}")
        json_write_file(policies_path, merged_app_policies_dict, 2)

    if get_os_name() == "Windows":
        ffox_policies_reg_path = r"SOFTWARE\Policies\Mozilla\Firefox"
        if reg_key_exists(HKEY_LOCAL_MACHINE, ffox_policies_reg_path):
            print(
                "WARNING: Registry policies has been removed because it bypasses the policies.json file (source: https://support.mozilla.org/bm/questions/1236197)."
            )
            registry_deletekey(HKEY_LOCAL_MACHINE, ffox_policies_reg_path, "", force=True)


def uninstall():
    for firefox_policies_dir in get_firefox_policies_dirs():
        policies_path = makepath(firefox_policies_dir, policies_file)

        # Write policies.json if needed
        if isfile(policies_path):
            app_policies_dict = json_load_file(policies_path)
            try:
                del app_policies_dict["policies"]["ExtensionSettings"][addon_id]
                print(f"Updating Firefox configuration to: {policies_path}")
                json_write_file(policies_path, app_policies_dict, 2)
            except KeyError as e:
                print(f"The key {addon_id} does not exist in the dictionary: {e}")

        for extension_file in glob.glob(makepath(firefox_policies_dir, extension_file_contains + "*.xpi")):
            remove_file(extension_file)


# def audit():
#     audit_status = "OK"
#     for firefox_policies_dir in get_firefox_policies_dirs():
#         policies_path = makepath(firefox_policies_dir, policies_file)
#         if not isfile(policies_path):
#             print("WARNING: Configuration is NOT applied for Firefox, re-applying...")
#             install()
#             audit_status = "WARNING"
#         else:
#             print("OK: Configuration is correctly applied for Firefox.")
#             audit_status = "OK"

#     return audit_status


def merge_dicts(dict1, dict2):
    for key, value in dict2.items():
        if key in dict1 and isinstance(value, dict) and isinstance(dict1[key], dict):
            merge_dicts(dict1[key], value)  # recursive merge
        else:
            dict1[key] = value
    return dict1
